////////////////////////////////////////////////////////////////////////////
// File:	MultiCtrl.h
// Version:	2
// Created:	05-Mar-2002
//
// Author:	Paul S. Vickery
// E-mail:	paul@vickeryhome.freeserve.co.uk
//
// Control switchable at run-time to become any one of a number of controls
//
// You are free to use or modify this code, with no restrictions, other than
// you continue to acknowledge me as the original author in this source code,
// or any code derived from it.
//
// If you use this code, or use it as a base for your own code, it would be 
// nice to hear from you simply so I know it's not been a waste of time!
//
// Copyright (c) 2001-2002 Paul S. Vickery
//
////////////////////////////////////////////////////////////////////////////
// Version History:
//
// Version 2 - 05-Mar-2002
// =======================
// Produced new version with changes as below:
// * added Rich Edit control to available control types
// * added support for ALT+<key> accelerators
// * improved check-box support - now takes lists of strings to use for each
//   of the check states
// * new public functions:
//   - BOOL Create(LPCTSTR lpszText, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID = 0xffff);
//   - virtual BOOL Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext = NULL);
//   - void CheckBoxUseLabel(BOOL bUseLabel = TRUE);
//   - void OffsetCheckbox(BOOL bOffset = TRUE);
//   - void SetCheckText(LPCSTR lpszChecked, LPCSTR lpszUnchecked, LPCSTR lpszIndeterminate = NULL);
//   - void SetCheckText(CStringList& slistChecked, CStringList& slistUnChecked, CStringList& slistIndeterminate);
//   - int GetCheckState(LPCSTR lpszValue);
//   - void SetEditSettings(int nStart, int nEnd, int nFirst);
//   - int GetEditSettings(int& nStart, int &nEnd);
//   - void Initialise();
//   - virtual CWnd* GetMainCtrl();
//   - BOOL IsValueChanged(LPCSTR lpszValue);
//   - CRichEditCtrl* GetRichEditCtrl();
//   - CButton* GetLabelButtonCtrl();
//   - void ModifyAttributes(int nAdd, int nRemove = 0);
//   - int AddListItems(CStringList& list);
//   - int AddListItem(LPCTSTR lpszText);
//
// Version 1 - 27-Jul-2001
// =======================
// Initial version
// 
////////////////////////////////////////////////////////////////////////////
// PLEASE LEAVE THIS HEADER INTACT
////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MULTICTRL_H__508EDC06_B649_11D4_A7D6_0000B48746CF__INCLUDED_)
#define AFX_MULTICTRL_H__508EDC06_B649_11D4_A7D6_0000B48746CF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// MultiCtrl.h : header file
//

#include "afxdtctl.h" // only needed for date/time controls
#include "BrowseEdit.h"

#include "HistoryCombo.h"

#define MCSLW_DEFAULT	(-1)
#define MCSLW_AUTO	(-2)

typedef enum
{
  MCT_NONE,		// empty control
  MCT_STATIC,		// static control
  MCT_STATICBTN,	// static control, with label as button
  MCT_EDIT,		// single edit control
  MCT_UPDOWNEDIT,	// edit with nested up/down control
  MCT_DROPLIST,		// drop-down list
  MCT_COMBO,		// combo box
  MCT_CHECKBOX,		// check box control
  MCT_DATETIME,		// date/time picker
  MCT_RICHEDIT,		// rich-edit control
  MCT_LAST
}MULTICTRLTYPE;

typedef enum
{
  MCA_NORMAL	= 0x00,	// normal - label visible, control visible and enabled
  MCA_READONLY	= 0x01,	// label visible and enabled, control visible but disabled
  MCA_DISABLED	= 0x02,	// everything visible but disabled
  MCA_HIDDEN	= 0x04,	// everything hidden
  MCA_NOLABEL	= 0x08,	// hide label
  MCA_BROWSEBTN	= 0x10, // include '...' button on the right of the control
  MCA_HISTORY	= 0x20, // combo box is a history combo
}MULTICTRLATTRIB;

/////////////////////////////////////////////////////////////////////////////
// CMultiCtrl control

class CMultiCtrl : public CStatic
{
	DECLARE_DYNAMIC(CMultiCtrl)
// Construction
public:
	CMultiCtrl();
	BOOL Create(LPCTSTR lpszText, DWORD dwStyle,
				const RECT& rect, CWnd* pParentWnd, UINT nID = 0xffff);
// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMultiCtrl)
	public:
	virtual BOOL Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext = NULL);
	protected:
	virtual void PreSubclassWindow();
	virtual BOOL OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult);
	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);
	virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
	//}}AFX_VIRTUAL

// Implementation
public:
  void CheckboxUseLabel(BOOL bUseLabel = TRUE);
  void OffsetCheckbox(BOOL bOffset = TRUE);
  void SetCheckText(LPCSTR lpszChecked, LPCSTR lpszUnchecked, LPCSTR lpszIndeterminate = NULL);
  void SetCheckText(CStringList& slistChecked, CStringList& slistUnchecked, CStringList& slistIndeterminate);
  int GetCheckState(LPCSTR lpszValue);
  void SetEditSettings(int nStart, int nEnd, int nFirst);
  int GetEditSettings(int& nStart, int& nEnd);
  void Initialise();
  virtual CWnd* GetMainCtrl();
  CString GetHelpString();
  void SetHelpString(LPCSTR lpszHelpString);
  BOOL IsValueChanged(LPCSTR lpszValue);
  void EnableControl(BOOL bEnable = TRUE);
  void ResetContent();
  CString GetTypeName();

  int GetLabelWidth(BOOL bAuto = FALSE);
  void SetLabelWidth(int nWidth = MCSLW_DEFAULT);

  CButton* GetButtonCtrl() { return m_pBtn; }
  CDateTimeCtrl* GetDateTimeCtrl() { return m_pDate; }
  CHistoryCombo* GetComboCtrl() { return m_pCombo; }
  CStatic* GetLabelStaticCtrl() { return m_pLabel; }
  CStatic* GetStaticCtrl() { return m_pStatic; }
  CEdit* GetEditCtrl() { return m_pEdit; }
  CSpinButtonCtrl* GetSpinCtrl() { return m_pSpin; }
  CRichEditCtrl* GetRichEditCtrl() { return m_pRichEdit; }
  CButton* GetLabelButtonCtrl() { return m_pLabelBtn; }

  int GetAttributes();
  void SetAttributes(int nAttrib);
  void ModifyAttributes(int nAdd, int nRemove = 0);

  int AddListItems(CStringList& list);
  int AddListItem(LPCTSTR lpszText);

  CString GetValueText() const;
  void SetValueText(LPCTSTR lpszValue);
  CString GetLabelText() const { return m_sLabelText; }

  MULTICTRLTYPE GetType();
  void SetType(MULTICTRLTYPE mct, DWORD dwStyleAdd = 0, DWORD dwStyleRemove = 0);

  void SetLabelText(LPCTSTR lpszString);
  int GetWindowText (LPTSTR lpszBuffer, int nMax) const;
  void GetWindowText (CString &rText) const;
  int GetWindowTextLength() const;
  void SetWindowText(LPCTSTR lpszText);
  void SetFont(CFont *pFont, BOOL bRedraw = TRUE);

  void SetTextLimit(UINT nMax);

  virtual ~CMultiCtrl();

protected:
  void SetStatusText(LPCSTR lpszText = NULL);
  CString m_sValueText;
  CString m_sHelpString;

  CStringList m_slistChecked;
  CStringList m_slistUnchecked;
  CStringList m_slistIndeterminate;
  BOOL m_bAllowIndeterminate;
  BOOL m_bOffsetCheckbox;
  BOOL m_bCheckboxUseLabel;

  // all possible child controls
  int m_nLabelWidth;
  CString m_sLabelText;

  UINT m_nTextLimit;

  CStatic* m_pLabel;
  CStatic* m_pStatic;
  CButton* m_pBtn;  // can be check-box, or '...' button
  CHistoryCombo* m_pCombo;
  CEdit* m_pEdit;
  CButton* m_pLabelBtn;
  CBrowseEdit* m_pBrowseEdit;
  CDateTimeCtrl* m_pDate;
  CSpinButtonCtrl* m_pSpin;
  CRichEditCtrl* m_pRichEdit;

  MULTICTRLTYPE m_mcType;
  int m_nAttrib;

  void PositionControls();

  BOOL CreateControls();
  BOOL CreateLabel();
  BOOL CreateLabelButton();
  BOOL CreateStatic();
  BOOL CreateCombo();
  BOOL CreateDropDown();
  BOOL CreateEdit();
  BOOL CreateBrowseEdit();
  BOOL CreateCheckbox();
  BOOL CreateButton();
  BOOL CreateDateTime();
  BOOL CreateSpin();
  BOOL CreateRichEdit();

  DWORD m_dwStyleAdd;
  DWORD m_dwStyleRemove;

  BOOL m_bInCreate;

  static int ms_nBaseUnitsX;
  static int ms_nBaseUnitsY;
  void CalcBaseUnits(BOOL bRecalc = FALSE);

	// Generated message map functions
	//{{AFX_MSG(CMultiCtrl)
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnEnable(BOOL bEnable);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg UINT OnGetDlgCode();
	//}}AFX_MSG
	afx_msg BOOL OnToolTipNotifyRange(UINT nID, NMHDR *pNMHDR, LRESULT *pResult);
	DECLARE_MESSAGE_MAP()

};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MULTICTRL_H__508EDC06_B649_11D4_A7D6_0000B48746CF__INCLUDED_)



